<?php
// $Id: pifr_server.result.inc,v 1.22 2009/12/04 20:29:07 boombatower Exp $

/**
 * @file
 * Provide test result functions.
 *
 * @author Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Load a result.
 *
 * @param integer $result_id Result ID.
 * @return array Result information as described below.
 * <code>
 *   $result = array(
 *     'result_id',
 *     'test_id',
 *     'environment_id',
 *     'code',
 *     'details',
 *     'data', // Data loaded by review plugin.
 *   );
 * </code>
 */
function pifr_server_result_get($result_id) {
  $result = db_query('SELECT *
                      FROM {pifr_result}
                      WHERE result_id = %d', $result_id);
  $result = db_fetch_array($result);
  $result['details'] = unserialize($result['details']);

  // Load plugin data.
  $result = pifr_server_review_load($result);

  return $result;
}

/**
 * Load the result associated with a test and environment.
 *
 * @param interger $test_id Test ID.
 * @param interger $environment_id Environment ID.
 * @return array Result information, or FALSE.
 * @see pifr_server_result_get()
 */
function pifr_server_result_get_environment($test_id, $environment_id) {
  $result = db_query('SELECT result_id
                      FROM {pifr_result}
                      WHERE test_id = %d
                      AND environment_id = %d', $test_id, $environment_id);
  if ($result_id = db_result($result)) {
    return pifr_server_result_get($result_id);
  }
  return FALSE;
}

/**
 * Load all the results for a test.
 *
 * @param integer $test_id Test ID.
 * @return array List results keyed by environment ID.
 * @see pifr_server_result_get()
 */
function pifr_server_result_get_all($test_id) {
  $result_ids = db_query('SELECT result_id
                          FROM {pifr_result}
                          WHERE test_id = %d', $test_id);

  $results = array();
  while ($result_id = db_result($result_ids)) {
    $result = pifr_server_result_get($result_id);
    $results[$result['environment_id']] = $result;
  }
  return $results;
}

/**
 * Determine the result if any that summarizes the test.
 *
 * Cycle through the results and finds the result that did not pass and thus
 * provides a summary of the review result.
 *
 * @param integer $test_id Test ID.
 * @return array Result information, or FALSE.
 * @see pifr_server_result_get()
 */
function pifr_server_result_summary_result($test_id) {
  $results = pifr_server_result_get_all($test_id);
  foreach ($results as $environment_id => $result) {
    if (!pifr_server_review_check($result)) {
      return $result;
    }
  }
  return FALSE;
}

/**
 * Generate a summary message based on the summary result.
 *
 * The summary result can be determined by pifr_server_result_summary_result().
 *
 * @param array $result Result information.
 * @return string Translated summary message.
 */
function pifr_server_result_summary_message($result) {
  if ($result) {
    $summary = pifr_server_review_summary($result);
    $details = pifr_server_review_summary_format($result);

    $environment = pifr_server_environment_get($result['environment_id']);
    $details['@environment'] = $environment['title'];
  }
  else {
    $summary = 'Passed on all environments.';
    $details = array();
  }

  return t($summary, $details);
}

/**
 * Clear all test result records (includes tree).
 *
 * @param integer $test_id Test ID.
 * @see pifr_server_result_get().
 */
function pifr_server_result_clear($test_id) {
  $results = pifr_server_result_get_all($test_id);
  foreach ($results as $result) {
    pifr_server_result_delete($result);
  }
}

/**
 * Save the specified result. The data will be saved in the three result
 * tables: pifr_result, pifr_result_detail, pifr_result_detail_assertion.
 *
 * @param array $result Result to save.
 * @return interger Result ID of saved result.
 */
function pifr_server_result_save(array $result) {
  drupal_write_record('pifr_result', $result);

  // Let plugins save additional data.
  pifr_server_review_record($result);

  return $result;
}

/**
 * Remove a test result.
 *
 * @param array $result Result information.
 */
function pifr_server_result_delete(array $result) {
  // Allow plugins to delete data before master result row is removed.
  pifr_server_review_delete($result);

  db_query('DELETE FROM {pifr_result} WHERE result_id = %d', $result['result_id']);
}
